# read csv,then info -> rdm
import os
import re
import logging
import matplotlib.pyplot as plt
from datetime import datetime
import csv
import configparser
from fuzzywuzzy import fuzz

class CSVToRdmInfo():
    def __init__(self, _path = '', _module2pic_config_path = './atcbaselib/logconfig.ini', _log_num_threshold_second = 60):
        self.path = _path
        self.module2pic_config_path = _module2pic_config_path #module2pic comfig path
        self.csv_path = [] #所有以.csv结尾的绝对路径
        self.log_times = {}   #{pic:log}
        self.boot_error_log = {}   #{pic:log}
        self.log_wrong_format = {} #{pic:log}
        self.log_num_threshold_second = _log_num_threshold_second
        self.log_times_rdm_info = {}
        self.boot_error_log_rdm_info = {}
        self.log_wrong_format_rdm_info = {}

    def find_csv(self):
        #遍历文件夹，找到.csv结尾的文件，存入self.csv_path
        for root, dirs, files in os.walk(self.path):
            for file in files:
                if str(file).endswith('.csv'):
                    csv_path = os.path.join(root, file)
                    self.csv_path.append(csv_path)

    def cal_similarity(self, log_dict, log):
        ratio = 0
        for key in log_dict:
            words_ori = key.split(' ')
            words_current = log.split(' ')
            ratio = fuzz.ratio(words_ori[1:], words_current[1:])
            if ratio > 85:
                return key, ratio
        return '', ratio

    def read_csv(self, keywords_list):
        self.find_csv()
        print(self.csv_path)
        all_csv = list(filter(lambda text: any([word in text for word in keywords_list]), self.csv_path))
        #是否将每个 csv的寻找pic过程变成异步线程？
        last_log = ""
        base_log = ""
        for csv_file in all_csv:
            csv_reader = csv.DictReader(open(csv_file))
            if "log_times" in csv_file:
                for row in csv_reader:
                    log = row['log']
                    pic = self.find_pic(log)
                    if pic == " ":
                        continue
                    if int(row['max_times_one_second']) < self.log_num_threshold_second:
                        continue
                    if pic in self.log_times:
                        key_log, radio = self.cal_similarity(self.log_times[pic], log)
                        if radio > 85:
                            last_log = row
                            base_log = key_log
                            continue
                        else:
                            self.log_times[pic].append(log)
                        # todo 检测log相似度，无论path，只看log
                    else:
                        self.log_times.setdefault(pic,[]).append(log)

            elif "log_wrong_format" in csv_file:
                for row in csv_reader:
                    log = row['log']
                    pic = self.find_pic(log)
                    if pic == " ":
                        continue
                    if pic in self.log_wrong_format:
                        key_log, radio = self.cal_similarity(self.log_wrong_format[pic], log)
                        if radio > 85:
                            last_log = row
                            base_log = key_log
                            continue
                        else:
                            self.log_wrong_format[pic].append(log)
                        #todo 检查log相似度，无论path，只看log
                    else:
                        self.log_wrong_format.setdefault(pic,[]).append(log)

            elif "boot_errorlog" in csv_file:
                for row in csv_reader:
                    log = row['log']
                    pic = self.find_pic(log)
                    if pic == " ":
                        continue
                    if pic in self.boot_error_log:
                        self.boot_error_log[pic].append(log)
                    else:
                        self.boot_error_log.setdefault(pic,[]).append(log)

            else:
                continue

    def find_pic(self, log):
        #logconfig.ini中，一级模块和二级模块是由空格进行分割的
        read_ini = configparser.ConfigParser()
        read_ini.read(self.module2pic_config_path, encoding='utf-8')
        logmodule2result = eval(read_ini['AC8025S-LogModule2PIC']['LOGMODULE2PIC'])
        for subteam in logmodule2result:
            for item in logmodule2result[subteam]:
                module_name_list = item.split(' ')
                if len(module_name_list) > 1:
                    first_module_name = item.split(' ')[0]
                    sub_module_name = item.split(' ')[1]
                    index = re.search(first_module_name + '.*' + sub_module_name, log)
                    if index == None:
                        continue
                    else:
                        return logmodule2result[subteam][item]
                else:
                    first_module_name = item.split(' ')[0]
                    index = re.search(first_module_name, log)
                    if index == None:
                        continue
                    else:
                        return logmodule2result[subteam][item]
        return ' '
        
    #传入common_info，组合成rdm的issues传出
    #common_info 包含的Key为  image_path、log_path、project、issue_type
    def prepareInfo(self, common_info):
        for item in self.log_times:
            self.log_times_rdm_info.setdefault(item,{})
            self.log_times_rdm_info[item]['image_path'] = common_info['image_path']
            self.log_times_rdm_info[item]['log_path'] = common_info['log_path']
            self.log_times_rdm_info[item]['summary'] = "[log check] max_times_one_second > 60"
            self.log_times_rdm_info[item]['issue_type'] = common_info['issue_type']
            self.log_times_rdm_info[item]['project'] = common_info['project']
            self.log_times_rdm_info[item]['pic'] = item
            self.log_times_rdm_info[item]['details'] = '<b>1. log_path: </b>' + common_info['log_path'] + ';</br>' \
                                                       '<b>2. Log check结果详情</b>: log_path/log_check_result/main_log_times.csv 或 kernel_log_times.csv;</br>' \
                                                        "<b>3. 部分具体logs: </b>"+ "</br>".join(self.log_times[item])
            self.log_times_rdm_info[item]['module'] = common_info['module']
        for item in self.boot_error_log:
            self.boot_error_log_rdm_info.setdefault(item,{})
            self.boot_error_log_rdm_info[item]['image_path'] = common_info['image_path']
            self.boot_error_log_rdm_info[item]['log_path'] = common_info['log_path']
            self.boot_error_log_rdm_info[item]['summary'] = "[log check] Error log appears at boot stage"
            self.boot_error_log_rdm_info[item]['issue_type'] = common_info['issue_type']
            self.boot_error_log_rdm_info[item]['project'] = common_info['project']
            self.boot_error_log_rdm_info[item]['pic'] = item
            self.boot_error_log_rdm_info[item]['details'] = '<b>1. log_path: </b>' + common_info['log_path'] + ';</br>' \
                                                            '<b>2. Log check结果详情</b>: log_path/log_check_result/boot_errorlog.csv;</br>' \
                                                            "<b>3. 部分具体logs:</b> "+ "</br>\t".join(self.boot_error_log[item])
            self.boot_error_log_rdm_info[item]['module'] = common_info['module']
        for item in self.log_wrong_format:
            self.log_wrong_format_rdm_info.setdefault(item,{})
            self.log_wrong_format_rdm_info[item]['image_path'] = common_info['image_path']
            self.log_wrong_format_rdm_info[item]['log_path'] = common_info['log_path']
            self.log_wrong_format_rdm_info[item]['summary'] = "[log check] Error log or warning log format error "
            self.log_wrong_format_rdm_info[item]['issue_type'] = common_info['issue_type']
            self.log_wrong_format_rdm_info[item]['project'] = common_info['project']
            self.log_wrong_format_rdm_info[item]['pic'] = item
            self.log_wrong_format_rdm_info[item]['details'] = '<b>1. log_path: </b>' + common_info['log_path'] + ';</br>' \
                                                              '<b>2. Log check结果详情</b>: log_path/log_check_result/log_wrong_format.csv;</br>' \
                                                              "<b>3. 部分具体logs: </b>"+ "</br>".join(self.log_wrong_format[item])
            self.log_wrong_format_rdm_info[item]['module'] = common_info['module']
    
    def do_prepare(self, common_info, keywords_list = ['log_times', 'log_wrong_format', 'boot_error_log']):
        logging.info('do prepare rdm info=======================')
        try:
            self.read_csv(keywords_list)
            self.prepareInfo(common_info)
        except Exception as a:
            print("prepare rdm info exception")
        


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO,
                        format='%(levelname)s: %(message)s -----%(filename)s(line:%(lineno)d):%(asctime)s')
    path = "D:/2022task/Log_Checker_Code/test/log/log"
    keywords_list=['log_wrong_format', 'boot_errorlog']
    common_info = {
        'image_path' : "\\172.28.133.21\Release\Sync\AC8025\Cockpit\cockpit-trunk-s.linux.ac8025\cockpit_s_linux_ac8025\minibuild\cockpit-trunk-s.linux.ac8025-AB-2023-02-17-02-00_cypress",
        'log_path' : '\\atcfs02\Release\AtcAutomatedTest\cockpit_s_linux_ac8025\integrationtest\cockpit_s_linux_ac8025-2023-02-17-11-46-59',
        'project': "AC8025SW",
        'issue_type' :'Unit Test',
        'module' : 'watermark'
    }
    rc = CSVToRdmInfo(path, 60)
    rc.do_prepare(keywords_list, common_info)
    print(rc.log_wrong_format_rdm_info)
    print(rc.log_times_rdm_info)
    print(rc.boot_error_log_rdm_info)

    rdm = CommomAutoRDM(host = "http://rdm.autochips.inc:8080")
    if rc.log_wrong_format_rdm_info:
        issues_result = rdm.create_bug(rc.log_wrong_format_rdm_info)
        for result in issues_result:
            print(result)
    if rc.log_times_rdm_info:
        issues_result = rdm.create_bug(rc.log_times_rdm_info)
        for result in issues_result:
            print(result)
    if rc.boot_error_log_rdm_info:
        issues_result = rdm.create_bug(rc.boot_error_log_rdm_info)
        for result in issues_result:
            print(result)

        

    





import logging
import requests
import pdb

#host = 'http://172.28.133.217:8080'  # test server
#host = 'http://rdm.autochips.inc:8080'
default_requst_info = {
    'login_url'    : '/user/login.wbs',
    'bug_url'      : '/startbug',
    'get_user_url' : '/user/get.wbs',
    'user_name'    : 'QAAuto',
    'password'     : 'zxcv!1234',
    'public_account'  : 'ci_build' 
}
class CommomAutoRDM():
    def __init__(self, request_info = default_requst_info, host = "http://172.28.133.217:8080", default_account = 'ATC0274'):
        self.host            = host
        self.default_account = default_account
        self.login_url       = request_info['login_url']
        self.bug_url         = request_info['bug_url']
        self.get_user_url    = request_info['get_user_url']
        self.user_name       = request_info['user_name']
        self.password        = request_info['password']
        self.public_account  = request_info['public_account']
        self.token = self.get_access_token()
        self.all_user_info = self.get_all_users_info()

    def get_all_users_info(self):
        url = "{}{}?token={}".format(self.host,  self.get_user_url, self.token)
        r = requests.request('GET', url)

        if 'error' in r.text:
            logging.error("get all users info fail : " + str(r.text))
            return None
        else:
            json_data = r.json()
            return json_data['User']

    def get_user_code_by_name(self, name):
        if self.all_user_info is not None:
            for info in self.all_user_info:
                if name.lower() in info["Email"].lower():
                    return info['Num']
        return self.default_account

    def get_access_token(self):
        url = "{}{}?userName={}&password={}".format(self.host, self.login_url, self.user_name, self.password)
        r = requests.request('GET', url)
        return r.text.rstrip()

    def get_project_key(self, project):
        if 'r.ac8015' in project:
            return 'AC8015R'
        elif 'p.ac8015' in project:
            return 'AC8015P'
        else:
            return 'AC8025SW'

    #input  : info包含的Key为 common_info 和 issues 
    #output : bug_num, bug_id, module和pic
    #   issues:{
    #         'image_path':,   
    #         'log_path':, 
    #         'summary': ,     # eg,unit test fail
    #         'issue_type': ,  # Unit Test
    #         'project':       # AC8025SW
    #         'pic':,
    #         'details':
    #         'module':
    #   }
    #
    #  description:
    #              log_path : test_log_path
    #              details  : issue['detail'] eg:watermarkwrapper.cpp line34 failed
    def create_bug(self, issues):
        logging.info('create rdm bug=======================')
        url = "{}{}".format(self.host, self.bug_url)
        issues_results = []

        for item in issues:
            body ={
                "token": self.token,
                "entity": {
                    "CreateCode": self.public_account,  # 创建人工号，必填
                    "OwnerCode": self.get_user_code_by_name(issues[item]['pic']),  # 责任人工号，必填
                    "ProjectCode": issues[item]['project'],  # 项目编号，必填
                    "Summary": issues[item]['summary'],
                    "IssueType": issues[item]['issue_type'],
                    "Platform": "Demo",
                    "Source": "RD",
                    "Project_Model": issues[item]['project'],
                    "Priority": "Medium",
                    "Severity": "Normal",
                    "Repeat_Steps": "No Repeat Steps",
                    "PICCode": self.get_user_code_by_name(issues[item]['pic']),
                    "SW_Version": issues[item]['image_path'],
                    "Expected_Due_Date": "",
                    "Description": issues[item]['details'],
                    "Repeat_Ratio": "Always",
                    "Bug_Category": "None"
                }
            }
            r = requests.request(method='post', url=url, json=body)
            logging.info('pic:' + issues[item]['pic'] +" rdm json:" + str(r.json()))
            if ('OK' in r.json()['Msg']):
                issue_msg = r.json()['BugCode']
                issue_id = r.json()['ObjectId']
            else:
                issue_msg = r.json()['Msg']
                issue_id = r.json()['ObjectId']
            result = {}
            result['pic'] = issues[item]['pic']
            result['bug_number'] = issue_msg
            result['bug_id'] = issue_id
            issues_results.append(result)
        return issues_results


# -*- coding: utf-8 -*-
import os
import re
import numpy as np
import logging
import matplotlib.pyplot as plt
from datetime import datetime
import csv
import re
from fuzzywuzzy import fuzz
import pdb

'''
    Log_Checker will find out below unsuitable behaviors in output logs:
        a) includes the error key words, such as '[E]', 'failed', 'err', etc
        b) some debug log output with un-acceptable frequence
        c) the log file lines exceed the system tolerance
        d) any other we want to filter out

        For item c), a simplfy method I used is sorting the top10 word.
           1) for normal log file, top10 word might fixed words, only 7~9 maybe unfixed.
           2) if a word suddenly rush into the rank, it will be a major suspector (assuming ranked into threhold top 5)
           3) if resonable after reviewing, we update the top10 histroy list we saved before

        Also, if we have a model that can get the characteristic of a sentence, for example we ignore some non-critical info of one sentence, we can find out the abnormal lines.
          Such as:
            "This is the testing log - 2021-7-8-22-12" and "This is the testing log - 2021-7-8-22-13", we can inference two sentences are dupulicated.
        I think it is more smart, accuratly, but it's diffcult for me to implement. So we can award the guys who implement this. :-)
'''

class Log_Time_Info():
    def __init__(self, _path = '', _log = ''):
        self.times = 0
        self.list_time = []
        self.freq = 0
        self.path = _path
        self.log = _log

    def set_last_time(self, time):
        self.list_time.append(time)
        self.times += 1
        if (self.times > 0 and (time - self.list_time[0]).seconds > 0):
            self.freq = self.times / (time - self.list_time[0]).seconds
        

class Log_Checker():
    def __init__(self, _log_path = "", _result_path = "", _uart_log_path = "uart_log.txt", _boot_log_path = "boot_log.txt", _top_word_path = "top.txt", _top_rank_num = 50, _log_total_num_thres = 1000, _log_num_thres_one_second = 20, _key_word_list = [" e ","fail","error","exception"]):
        logging.info("============Log_Check Init====================")
        self.log_path         = _log_path           # Log file path we want to analysis
        self.boot_log_path    = _boot_log_path      # boot log path
        self.uart_log_path    = _uart_log_path      # uart log path
        self.top_word_path    = _top_word_path      # Project top word rank list
        self.result_path      = _result_path
        self.log_line_num     = 0                   # Total lines number
        self.log_total_num_thres   = _log_total_num_thres 
        self.log_num_thres_one_second = _log_num_thres_one_second
        self.top_rank_num     = _top_rank_num
        self.key_word_list    = _key_word_list

        self.pattern_dic      = {}                  # Save lines which including the pattern world
        self.word_dic         = {}                  # Save the files words count
        self.cur_top_rank     = {}                  # Current log file top rank
        self.rushman_dic      = {}
        self.all_log_path = {}

        if os.path.exists("top_rank.npy"):
            self.ori_top_rank = np.load("top_rank.npy", allow_pickle=True).item()  # Original top rank
        else:
            self.ori_top_rank = None

        logging.debug('top_rank: {}'.format(self.ori_top_rank))
        for w in self.key_word_list:
            self.pattern_dic[w] = []                # Init the pattern dic

        logging.info("Dump reg info:")
        logging.info(self.pattern_dic)                     # Dump debug information

    def find_rushman(self):
        logging.info("============Find_rushman======================")
        rushman = []

        if self.ori_top_rank is None:
            logging.info('No rank! exit')
            return

        i = 0
        logging.debug(self.ori_top_rank.keys())
        for m in self.cur_top_rank.keys():
            if m not in self.ori_top_rank.keys():
                rushman.append( m )
                i = i + 1

        if len(rushman) == 0:
            logging.info("All Acquaintance!!!!!")
        else:
            logging.info("Catch the rushman :" + str(rushman))

            for n in rushman:
                i = 0
                self.rushman_dic.setdefault(n, 0)
                for l in self.cur_top_rank.keys():
                    if n == l:
                        break
                    i = i + 1
                self.rushman_dic[n] = i
        return

    def cal_similarity(self, log_dict, log):
        ratio = 0
        for key in log_dict:
            words_ori = key.split(' ')
            words_current = log.split(' ')
            ratio = fuzz.ratio(words_ori[1:], words_current[1:])
            if ratio > 85:
                return key, ratio
        return '', ratio

    def find_log_path(self, root, pattern):
        dirs = []
        for x in os.listdir(root):
            nd = os.path.join(root, x)
            if os.path.isdir(nd):
                dirs.append(nd)
            elif os.path.isfile(nd) and 'uart' in pattern and 'uart_log.txt' in nd:
                    self.all_log_path.setdefault('uart',[]).append(nd)
            elif os.path.isfile(nd) and "adas" in root:
                if 'adas' in pattern and 'syslog' in nd:
                    self.all_log_path.setdefault('adas',[]).append(nd)
            elif os.path.isfile(nd) and "cluster" in root:
                if 'cluster' in pattern and 'syslog' in nd:
                    self.all_log_path.setdefault('cluster',[]).append(nd) 
            elif os.path.isfile(nd) and 'ivi' in root:
                if 'main' in pattern and 'main' in nd:
                    self.all_log_path.setdefault('main',[]).append(nd)
                elif 'kernel' in pattern and 'kernel' in nd:
                    self.all_log_path.setdefault('kernel',[]).append(nd)
        for dir in dirs:
            self.find_log_path(root=dir, pattern=pattern)

    # input log_line
    # output log_time, module_name, main_log
    def parse_log(self, log_line, pattern):
        if "adas" in pattern or "cluster" in pattern or "uart" in pattern:
            res = re.findall('\[[A-Z]\]', log_line)
            if res:
                log_level = res[0]
            else:
                return None, None, None
            start = log_line.find(log_level) + 3
            main_log = log_line[start:].strip()
            start_iter = re.search('\[[a-zA-Z]', main_log)#eg:[151][151][[AVMViewManager]] -> [AVMViewManager]
            module = re.findall('\[.*\]', main_log[start_iter.span()[0]:] if start_iter else main_log[0:])
            module_name = ""
            if len(module) >= 1:
                module_name = re.sub('\[|\]','', module[0])
            try:
                if "uart" in pattern:
                    log_time = datetime.strptime(str(log_line[1:20]), "%Y-%m-%d-%H-%M-%S")
                else:
                    tmp = log_line.split('  ')
                    timestamp = tmp[0] + " " + " ".join(tmp[1].split(' ')[0:2])
                    log_time = datetime.strptime(timestamp.split(".")[0], "%b %d %H:%M:%S")
            except Exception as a:
                return None, None, None
            return log_time, module_name, main_log

        elif "main" in pattern:
            res = re.findall('[A-Z]+', log_line)
            if res:
                log_level = res[0]
            else:
                return None, None, None
            try:
                start = log_line.find(log_level) + 1
                main_log = log_line[start:].strip()
                start_iter = re.search('\[[A-Z]', main_log)#eg:[151][151][[AVMViewManager]] -> AVMViewManager
                module = re.findall('\[.*\]', main_log[start_iter.span()[0]:] if start_iter else main_log.split(":")[0])
                module_name = ""
                if len(module) >= 1:
                    module_name = re.sub('\[|\]','', module[0])
                if module_name == "":
                    module_name = main_log.split(':')[0]
                log_time = datetime.strptime(str(log_line[0:14]), "%m-%d %H:%M:%S")
            except Exception as a:
                return None, None, None
            return log_time, module_name, main_log

        elif "kernel" in pattern:
            start = log_line.rfind(']') + 1
            main_log = log_line[start:].strip()
            try:
                start_iter = re.search('\[[A-Z]', main_log)#eg:[151][151][[AVMViewManager]] -> AVMViewManager
                module = re.findall('\[.*\]', main_log[start_iter.span()[0]:] if start_iter else main_log.split(":")[0])
                module_name = ""
                if len(module) >= 1:
                    module_name = re.sub('\[|\]','', module[0])
                if module_name == "":
                    module_name = main_log.split(':')[0]
                log_time = datetime.strptime(str(log_line[0:14]), "%m-%d %H:%M:%S")
            except Exception as a:
                return None, None, None
            return log_time, module_name, main_log

        return None, None, None

    def check_log_times(self, patterns = ['main', 'adas', 'cluster', 'kernel']):
        for pattern in patterns:
            result_log_info = {}
            last_log = ""
            base_log = ""
            result_file_name = self.result_path + '/' + pattern + '_log_times.csv'
            logging.info("============check_" + str(pattern)+ "_log_times====================")
            logging.info("log path : " + self.log_path)
            self.find_log_path(self.log_path, pattern)
            logging.info(self.all_log_path[pattern])

            for path in self.all_log_path[pattern]:
                try:
                    with open(path, "r", encoding='ISO-8859-1') as f:
                        print(path)
                        all_log = f.readlines()
                except Exception as a:
                    print(a)
                    continue
                for log_line in all_log:
                    #find the log level and log module name
                    log_time, module_name, main_log = self.parse_log(log_line, pattern)
                    if not log_time and not module_name and not main_log:
                        continue

                    key = path + main_log
                    if module_name in result_log_info:
                        if key in result_log_info[module_name]:
                            if last_log == key:
                                result_log_info[module_name][base_log].set_last_time(log_time)
                            else:
                                result_log_info[module_name][key].set_last_time(log_time)
                        else:
                            key_log, radio=self.cal_similarity(result_log_info[module_name], main_log)
                            if radio > 85 and result_log_info[module_name][key_log].path == path:
                                result_log_info[module_name][key_log].set_last_time(log_time)
                                last_log = key
                                base_log = key_log
                            else:
                                info = Log_Time_Info(path, main_log)
                                info.set_last_time(log_time)
                                result_log_info[module_name][key] = info
                    else:
                        result_log_info[module_name] = {}
                        info = Log_Time_Info(path, main_log)
                        info.set_last_time(log_time)
                        result_log_info[module_name][key] = info
            self.write_times_check_result(result_log_info, result_file_name)

    def write_times_check_result(self, log_dict, result_file):
        with open(result_file, 'w', newline='') as csvfile:
            fieldnames = ['log', 'path', 'times', 'freq', 'max_time', 'max_times_one_second']
            writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
            writer.writeheader()
            for module_name in log_dict:
                for info in log_dict[module_name]:
                    max_time = max(log_dict[module_name][info].list_time, key=log_dict[module_name][info].list_time.count)
                    max_times_one_second = log_dict[module_name][info].list_time.count(max_time)
                    if log_dict[module_name][info].times > int(self.log_total_num_thres) or max_times_one_second > int(self.log_num_thres_one_second):
                        writer.writerow({'log': log_dict[module_name][info].log, 'path': log_dict[module_name][info].path.replace('log_for_check',''),
                                        'times': log_dict[module_name][info].times, 'freq': log_dict[module_name][info].freq,
                                        'max_time': max_time, 'max_times_one_second': max_times_one_second})
        csvfile.close()

    def check_log_format(self):
        logging.info("============Check_Log_Format====================")
        LIST_FILE_SUFFIX = [".cpp", ".java", ".c"]
        LIST_LOG_LEVEL = ["[E]"]

        containsAny = lambda seq, aset: True if any(i in seq for i in aset) else False

        wrong_format_log = []
        with open(self.uart_log_path, "r") as f:
            all_log = f.readlines()

        for log_line in all_log:
            if containsAny(log_line, LIST_LOG_LEVEL) and not containsAny(log_line, LIST_FILE_SUFFIX):
                wrong_format_log.append(log_line.strip())

        with open(self.result_path + '/log_wrong_format.csv', 'w', newline='') as csvfile:
            fieldnames = ['log']
            writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
            writer.writeheader()
            for log in wrong_format_log:
                writer.writerow({'log': log})
            csvfile.close()

    def find_elog_at_boot(self):
        logging.info("============find_E_log====================")
        with open(self.boot_log_path, "r") as f:
            all_log = f.readlines()

        elog = [x.strip() for x in all_log if '[E]' in x]

        if len(elog) == 0:
            logging.info("not E log at boot")
            return

        with open(self.result_path + '/boot_errorlog.csv', 'w', newline='') as csvfile:
            fieldnames = ['log']
            writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
            writer.writeheader()
            for log in elog:
                writer.writerow({'log': log})
            csvfile.close()

    def check_duration_warn_log(self):
        logging.info("============Check_duration_warn_Log====================")
        duration_warn_log = []
        self.find_log_path(self.log_path, 'kernel')
        print(self.all_log_path['kernel'])

        for path in self.all_log_path['kernel']:
            with open(path, "r", encoding='ISO-8859-1') as f:
                print(path)
                try:
                    all_log = f.readlines()
                except Exception as a:
                    print(a)
                    continue

            for log_line in all_log:
                if 'DURATION WARN' in log_line:
                    info = Log_Time_Info(path, log_line.strip())
                    duration_warn_log.append(info)

        if len(duration_warn_log) == 0:
            logging.info("not duration warn log")
            return

        with open(self.result_path + '/duration_warn_log.csv', 'w', newline='') as csvfile:
            fieldnames = ['log', 'path']
            writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
            writer.writeheader()
            for warn_log in duration_warn_log:
                writer.writerow({'log': warn_log.log, 'path': warn_log.path.replace('log_for_check','')})
            csvfile.close()

    def do_top_rank(self):
        logging.info("============Find_suspector====================")
        if os.path.exists(self.top_word_path):
            with open(self.top_word_path, 'r') as lf:
                text = lf.read()
                temp_words = re.split('[^a-zA-Z]', text)      # Split the words from the line
                words = list(filter(None, temp_words))        # Give up all None string

                for word in words:
                    self.word_dic.setdefault(word, 0)
                    self.word_dic[word] = self.word_dic[word] + 1  # Calculate the specail word's count
                lf.close()
        else:
            logging.info("top rank file not exited")
            return

        # Sort the dic from big to small by the dic value
        sorted_list = (sorted(self.word_dic.items(), key = lambda kv:(kv[1], kv[0]), reverse=True))

        for i in range(self.top_rank_num):
            # Save the top N words  0:(word, count)
            self.cur_top_rank[sorted_list[i][0]] = sorted_list[i][1]

        #np.save("top_rank.npy", self.cur_top_rank)

    def find_abnormal_sentences(self):
        logging.info("============Find_abnormal_sentences and total line number===========")
        if os.path.exists(self.log_path):
            with open(self.uart_log_path, 'r') as lf:
                while True:
                    line = lf.readline()
                    for w in self.key_word_list:
                        if w in line.lower():
                            self.pattern_dic[w].append(line)
                    self.log_line_num = self.log_line_num + 1
                    if line == '':
                        break

                lf.close()

    def do_check(self):
        logging.info("Do log check")
        self.find_abnormal_sentences()
        self.do_top_rank()
        self.find_rushman()

    def gen_result(self, result_filename):
        x_data = []
        y_data = []
        logging.info(self.result_path + result_filename)
        for m in self.pattern_dic:
            logging.info(str(m) + ": " + str(len(self.pattern_dic[m])))
            y_data.append(len(self.pattern_dic[m]))
            x_data.append(m)

        fig = plt.figure(figsize=(10,10))

        data_cnt = len(self.pattern_dic)
        color = ['#A9A9A9','#006699', '#006400', '#999933','#CC9966', '#FFFFCC']
        plt.bar(x_data, y_data, width=0.5, color = color[1])
        for a,b in zip(x_data, y_data):
            plt.text(a,b,
                     b,
                     ha='center', 
                     va='bottom',
                    )
        ax = plt.gca()
        ax.set_xticks(range(data_cnt))
        ax.set_xticklabels(x_data, rotation = 45)
        ax.set_yticks([0,512,3072,3584,4096])
        ax.set_xlabel("key word")
        ax.set_ylabel("The number of key word")
        ax.set_title("log_check") 
        plt.savefig(self.result_path + result_filename)
        #plt.show()

    def report(self):
        # Generate the report file
        logging.info("============Report result================")
        #logging.debug("============All sentence need to be reviewed:")
        #for m in self.pattern_dic:
        #    logging.debug(str(m) + ": \n")
        #    for n in self.pattern_dic[m]:
        #        logging.debug("\t" + n)

        # print top rank on screen
        logging.info("============Top rank status:")
        logging.info("\t" + str(self.cur_top_rank))

        #
        logging.info("============Total line number: ")
        logging.info("\tlog line number: " + str(self.log_line_num))
        logging.info("\tline number threshold: " + str(self.line_num_thres))

        #
        logging.info("============High Risk words: ")
        logging.info("\t" + str(self.rushman_dic))


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO,
                        format='%(levelname)s: %(message)s -----%(filename)s(line:%(lineno)d):%(asctime)s')
    path = "D:/2022task/Log_Checker_Code/test/log/log"
    line_num_threshold = input("每秒输出最大次数")
    print(line_num_threshold)
    key_word_list = [" e ","fail","error","exception"]
    top_rank_num = 50

    lc = Log_Checker(path, path, key_word_list, path + "/uart_log.txt", path + "/boot_log.txt", "top.txt", 50, 1000, line_num_threshold)
    lc.do_check()
    lc.find_elog_at_boot()
    lc.check_log_format()
    lc.gen_result("result.png")
    if (os.path.exists(path)):
        lc.check_duration_warn_log()
        lc.check_log_times(['main', 'adas', 'cluster', 'kernel'])





from setuptools import setup, find_packages
setup(name='atcbaselib',
      version='0.0.3',
      description='Log check and auto rdm bug',
      author='zw',
      author_email='zwwwww@163.com',
      requires= ['numpy','matplotlib', 're', 'os', 'datetime', 'csv', 'fuzzywuzzy'], # 定义依赖哪些模块
      packages=find_packages(),  # 系统自动从当前目录开始找包
      # 如果有的文件不用打包，则只能指定需要打包的文件
      #packages=['代码1','代码2','__init__']  #指定目录中需要打包的py文件，注意不要.py后缀
      license="apache 3.0",
      data_files=['atcbaselib/logconfig.ini']
      )